(*Description...
MultiSystem V1.1 is a framework for integrating different scripts and running them as one system.

This script contains one of the sample systems, based on rickty's "Stochastic price reverse - market V4.5".

This script is not a trading system in itself, but rather a tool for developing more sophisticated  trading systems. MultiSystem compatible scripts cannot be run on the web site, as the include directive {$I } is only supported by the Wealth-Lab Developer, so you will need to download all the components on the desktop and run them from there.

Click <b><a href="http://home.pacbell.net/michel/multisystem_v1_1_help.htm">here</a></b> for the complete documentation.

These are the different components of the MultiSystem set of scripts:
<b>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18706">MultiSystem V1.1</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18731">MultiSystem V1.1 Positions</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18708">MultiSystem V1.1 FX</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18710">MultiSystem V1.1 MasterSample</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18711">MultiSystem V1.1 Sample System1</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18712">MultiSystem V1.1 Sample System2</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18713">MultiSystem V1.1 Sample System3</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18714">MultiSystem V1.1 System Template</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18715">MultiSystem V1.1 Filtered System</a>
</b>

The sample provided also uses these utility scripts, so they will have to be present in the system in order for the sample to run without errors:

<b><a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18716">File</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18717">SymbolsList</a>
</b>

Here are links to the previous versions of the MultiSystem set of scripts 
<b><a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=14774">MultiSystem V1.0 </a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=14912">MultiSystem V1.0.1 </a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=15040">MultiSystem V1.0.2 </a></b>

*)
//**********************************
//
// MultiSystem V1.1
//
// System Sample1 - based on rickty's "Stochastic price reverse - market V4.5"
//
// Adrian Michel - 2003
//**********************************

type System1 = class( FilteredSystem )

private
  STOCHPANE, BAR: integer;
  StochPeriod, StochPeriodShort: integer;

public constructor Create;
protected procedure runBar( Bar : integer );override;
end;


constructor System1.Create;
begin
inherited Create( 20, 'Stochastic price reverse - market V4.5 - sample' );
{Stoch. period for longs}
StochPeriod:= 9;

{Stoch. period for shorts}
StochPeriodShort:= 16;
end;

procedure System1.runBar( Bar : integer );
begin
    if not LastPositionActive then
    {Entry rules}
     begin
       if (StochK( Bar, StochPeriod) < 5)  then
         if (0.96*@#AverageC[ Bar-2 ] > @#AverageC[ Bar-1 ] )  then
           if (0.86*@#AverageC[ Bar-1 ] > @#AverageC[ Bar ] )  then
              BuyAtMarket(Bar+1,'');
         
       if (StochK( Bar, StochPeriodShort) > 97) then
         if (1.12*@#AverageC[ Bar-2 ] < @#AverageC[ Bar-1 ] )  then
           if (1.24*@#AverageC[ Bar-1 ] < @#AverageC[ Bar ] )  then
              ShortAtMarket(Bar+1,'');
     end
     else
     {Exit rules}
     begin
       if MarketPosition = 1 then SellAtClose( Bar + 1, LastPosition, '' );
       if MarketPosition = -1 then CoverAtClose( Bar + 1, LastPosition, '' );
     end;
end;
