(*Description...
MultiSystem V1.1 is a framework for integrating different scripts and running them as one system.

This script contains one of the sample systems, based on dmacdonal9's "RSI with ADX filter & ATR Exit" enhanced to allow for filtering (see the buy and short lines, which are now conditional).

This script is not a trading system in itself, but rather a tool for developing more sophisticated  trading systems. MultiSystem compatible scripts cannot be run on the web site, as the include directive {$I } is only supported by the Wealth-Lab Developer, so you will need to download all the components on the desktop and run them from there.

Click <b><a href="http://home.pacbell.net/michel/multisystem_v1_1_help.htm">here</a></b> for the complete documentation.

These are the different components of the MultiSystem set of scripts:
<b>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18706">MultiSystem V1.1</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18731">MultiSystem V1.1 Positions</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18708">MultiSystem V1.1 FX</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18710">MultiSystem V1.1 MasterSample</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18711">MultiSystem V1.1 Sample System1</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18712">MultiSystem V1.1 Sample System2</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18713">MultiSystem V1.1 Sample System3</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18714">MultiSystem V1.1 System Template</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18715">MultiSystem V1.1 Filtered System</a>
</b>

The sample provided also uses these utility scripts, so they will have to be present in the system in order for the sample to run without errors:

<b><a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18716">File</a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18717">SymbolsList</a>
</b>

Here are links to the previous versions of the MultiSystem set of scripts 
<b><a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=14774">MultiSystem V1.0 </a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=14912">MultiSystem V1.0.1 </a>
<a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=15040">MultiSystem V1.0.2 </a></b>

*)
//**********************************
//
// MultiSystem V1.1
//
// System Sample2 - based on dmacdonal9's "RSI with ADX filter & ATR Exit"
//        Enhanced to allow for filtering (see the buy and short lines, which are now conditional)
//
// Adrian Michel - 2003
//**********************************

type System2 = Class( FilteredSystem )
private
 atrprofit, atrstop, nextatrprofit, temp, thisatr, thisrsi, buyprice,
    limitprice, exitprice, maxrisk, maxriskprice : float;
 atrrange,srsi,lrsi,rsiperiod,minprice,minadx,adxperiod,
    Bar, buybar, p  : integer;
 str : string;

public
public constructor Create;
protected procedure runBar( Bar : integer );override;
end;


constructor System2.Create;
begin
inherited Create( 41, 'RSI with ADX filter & ATR Exit - sample' );
{ setup variables }
atrprofit := 200 / 100;        { percentage of the ATR as a target}
nextatrprofit := 110 / 100;    { target for day 2, calculated from the open }
atrstop := 100 / 100;          { percentage of the ATR as a stop }
maxrisk := 15;                 { never more than this, even on big atr stocks }
atrrange := 21;                { need lots of bars to establish TRUE range }
lrsi := 18;
srsi := 82;
rsiperiod := 5;                { this seems to suit the short term hold period }
minprice := 0;                 { play with price levels here }
minadx := 22;                  { filter out rangebound stocks }
adxperiod := 14;
end;

procedure System2.runBar( Bar : integer );
begin
  { let's calc this for later }
  thisrsi := RSI(Bar,#close,rsiperiod);
  thisatr := ATR(Bar,atrrange);

  for p := 0 to PositionCount() - 1 do
  begin

     if PositionActive(p) then
     begin

       buybar := PositionEntryBar(p);
       buyprice := PositionEntryPrice(p);

       if PositionLong( p ) then
       begin

         { Use our wide ATR exits on day one }
         if ( Bar = buybar) then
         begin
           limitprice := buyprice + (atrprofit * thisatr);
           exitprice := buyprice - (atrstop * thisatr);
           maxriskprice := buyprice * (1 - (maxrisk / 100));
           if (exitprice < maxriskprice) then exitprice := maxriskprice;

           if not SellAtStop( Bar, exitprice, p, 'atrstop') then
   	       if (PriceClose(Bar) > limitprice) then
               SellAtClose( Bar, p, 'day1close');
         end;

         { Tighten the target, and get out at the close if we don't get it }
         if ((Bar - buybar) = 1)
         AND (PositionActive(p)) then
         begin
           limitprice := PriceOpen(Bar) + (nextatrprofit * thisatr);
           exitprice := PriceLow(Bar-1) * 0.995;
           maxriskprice := buyprice * (1 - (maxrisk / 100));

           if (exitprice < maxriskprice) then exitprice := maxriskprice;

           if not SellAtStop( Bar, exitprice, p, 'day2stop') then
             if not SellAtLimit( Bar, limitprice, p, 'day2limit') then
               SellAtClose(Bar,p,'timeout');
         end;

       end
       else
       begin

         if ( Bar = buybar ) then
         begin
           buyprice := PositionEntryPrice(p);
           limitprice := buyprice - (atrprofit * thisatr);

           exitprice := buyprice + (atrstop * thisatr);
           maxriskprice := buyprice * (1 + (maxrisk / 100));
           if (exitprice > maxriskprice) then exitprice := maxriskprice;

           if not CoverAtStop( Bar, exitprice, p, 'atrstop') then
             if (PriceClose(Bar) < limitprice) then
               CoverAtClose( Bar, p, 'day1close');
         end;

         if ((Bar - buybar) = 1)
         AND (PositionActive(p)) then
         begin
           limitprice := PriceOpen(Bar) - (nextatrprofit * thisatr);
           exitprice := PriceHigh(Bar-1) * 1.005;
           maxriskprice := buyprice * (1 + (maxrisk / 100));
           if (exitprice > maxriskprice) then exitprice := maxriskprice;

           if not CoverAtStop( Bar, exitprice, p, 'day2stop') then
             if not CoverAtLimit( Bar, limitprice, p, 'day2limit') then
               CoverAtClose(Bar,p,'timeout');
         end;
       end;
     end;
   end;  { for position loop }

   { entry rules... }

   if (ADX(bar,adxperiod) > minadx)
   AND (thisrsi < lrsi)
   AND (Pricelow(bar) > minprice)
   then
   begin
     if( BuyAtMarket( Bar+1,'') ) then
     { tell simulator to enter more oversold stocks first }
         SetPositionData(LastPosition, (100 - thisrsi));
   end;


   if ((ADX(bar,adxperiod) > minadx)
   AND (thisrsi > srsi)
   AND (PriceLow(bar) > minprice))
   then
   begin
     if( ShortAtMarket( Bar+1, '') ) then
     { tell simulator to enter more overbought stocks first }
        SetPositionData(LastPosition, thisrsi);
   end;
end;

{var INDPane: integer;
INDPane := CreatePane( 50, true, true );
PlotSeries( RSISeries( #Close,rsiperiod ), INDPane, 009, #Thin );
str := 'ATR(' + FloatToStr(atrrange) + ') = ' + FormatFloat('##.##',thisatr);
DrawLabel( str, INDPane );
str := 'RSI(5): ' + FormatFloat('##.##',thisrsi);
DrawLabel( str, INDPane );
}
