(*Description...
This is a utility script and does not implement a trading system. It is currently used as part of the <b><a href="http://www.wealth-lab.com/cgi-bin/WealthLab.DLL/editsystem?id=18706">MultiSystem V1.1</a></b> sample system

Loads a list of symbols from a file and gives access to individual symbols used to filter by symbol in MultiSystem. 

Usage: 
- create a SymbolList object by passing the constructor the file name, a local name, and the separator character used to separate symbols on the same line.
- get the number of symbols using Count;
- get the symbol at index using getSym( index );
- test for a symbol presence using isInSymbolList( symbol ); 
- see contents of the object using dump - sends the contents to the debug window

*)
//*******************************************
// SymbolsList class
//
// Adrian Michel 2003
//
// loads a list of symbols from a file and gives access to individual symbols
// used to filter by symbol in MultiSystem.
//
// Usage: create a SymbolList object by passing the constructor the file name and a local name,
//   and a separator character used to separate different symbols on the same line in the
//   symbols file.
// get the number of symbols using Count;
// get the symbol at index using getSym( index );
// test for a symbol presence using isInSymbolList( symbol );
// see contents of the object using dump - sends the contents to the debug window
//
// depends on the script File
//*******************************************

{$I 'File'}

type SymbolsList = class
     private _name : String;
     private _fileName : String;
     private _symbols : TList;
     private _sep : String;
     
       // optimzed the search in case of repeated search of the same symbol,
       // which is often the case for MultiSystem
     private _lastSymbol : string;
     private _lastCompare : boolean;

     public constructor Create( fileName, name, separator : String );

     public function isInSymbolsList( symbol : String ) : boolean;
     public function count : integer;
     public function getSym( index : integer ) : String;
     private procedure addLine( l : string );
     public procedure dump;
end;

procedure SymbolsList.addLine( l : string );
begin
     var token : string;
     var index : integer;
     index := 0;
     
     while true do
     begin
         token := GetToken( l, index, _sep );
         if( Length( token ) > 0 ) then
         begin
             _symbols.Add( token );
             index := index + 1;
         end
         else
             break;
     end
end;

constructor SymbolsList.Create( fileName, name, separator : String );
begin
     _sep := separator;
     _symbols := TList.Create;
     _name := name;
     _fileName := fileName;

     var _file : File;
     // TODO: how to check that the file exists and not create a new one?
     _file := File.Create( fileName, MODE_OPEN );
     var l : string;
     while( not _file.Eof ) do
     begin
          l := _file.readLine;
          addLine( l );
     end;
     _lastSymbol := '';
     _lastCompare := false;
end;

function SymbolsList.isInSymbolsList( symbol : String ) : boolean;
begin
     if( symbol = _lastSymbol ) then
     begin
         Result := _lastCompare;
     end
     else
     begin
         // does a linear search - should do a binary search (sort the list first).
         var index : integer;
         var str : String;
         Result := false;
         _lastCompare := false;
     
         for index := 0 to _symbols.Count - 1 do
         begin
             str := _symbols.Item( index );
             if( CompareText( str, symbol ) = 0 ) then
             begin
                 Result := true;
                 _lastCompare := true;
                 break;
             end;
         end;
         _lastSymbol := symbol;
     end
end;

function SymbolsList.Count : integer;
begin
     Result := _symbols.Count;
end;

function SymbolsList.getSym( index : integer ) : string;
begin
//TODO: make sure index is within bounds, should this be an assert? for now return an empty string
//     assert( index < _symbols.Count );
       if( index >= _symbols.Count ) then
       begin
           Result := '';
       end
       else
           Result := _symbols.Item( index );
end;

procedure SymbolsList.dump;
begin
     Print( 'contents of SymbolsList: ' + _fileName );
     var index : integer;
     for index := 0 to _symbols.Count - 1 do
     begin
          Print( _symbols.Item( index ) );
     end;
end;

{
//********************************
// test
var sl : SymbolsList;
sl := SymbolsList.Create( 'd:\4.txt', 'list', ' ' );
sl.dump;
}
